% read and process sonic anemometer data, recorded via hyperterminal (no timestamps other than filename)

clear all, close all
warning('off','stats:statrobustfit:IterationLimit')


%fpath = '/Users/mike/Documents/UW_Year_1/Research/Data/'
fpath = '/Volumes/Data/JdFwaves/Robertson/'


rate = 10;  % sampling rate (Hz)
fmin = 1;  % inertial sub-range
fmax = 4; % inertial sub-range

files = dir([fpath 'Robertson_SonicAnemometer*.TXT']);

for fi = 1:length(files), 
    
    files(fi).name
    clear raw* epsilon* sigma
    figure(1), clf
    
    %% read in
    
    filestarttime = datenum(  [ files(fi).name(27:28) files(fi).name(29:31) files(fi).name(32:35) ] )  + str2num([files(fi).name(37:38)])./24 + str2num([files(fi).name(39:40)])./(24*60);
    fileendtime = datenum( files(fi).date );
    
    [ u v w err temp ] = textread([fpath files(fi).name],'%n%n%n%s%n','delimiter',' ');%,'headerlines',5);
    % note these components are relative to the boat
    
    %% quality control (err code is cell, but temp is double and easier to use)
    %bad = find(temp<0);
    %u(bad) = NaN;
    %v(bad) = NaN;
    %w(bad) = NaN;
    
    %% despike
    [ u v w spikeinds] = func_despike_phasespace3d_3var( u, v, w, 2 );
    nbadpts = length(spikeinds);
    
    %% reshape into bursts 
    
    n = length(u);
    burstlength = 5*60;  % secs
    burstpts = rate*burstlength;  % pts 
    bursts = floor(n./burstpts);  %number of bursts
    
    %u = reshape(u([1:(n-rem(n,burstpts))]), bursts ,burstpts);
    %v = reshape(v([1:(n-rem(n,burstpts))]), bursts ,burstpts);
    %w = reshape(w([1:(n-rem(n,burstpts))]), bursts ,burstpts);
    u = reshape(u([1:(n-rem(n,burstpts))]), burstpts ,bursts)';
    v = reshape(v([1:(n-rem(n,burstpts))]), burstpts ,bursts)';
    w = reshape(w([1:(n-rem(n,burstpts))]), burstpts ,bursts)';
    
    time = linspace(filestarttime,fileendtime,bursts);
    
    
    %% stats
    
    rawWindSpd = sqrt( u.^2 + v.^2 );
    rawmeanWindSpd = nanmean( rawWindSpd, 2 ); 
    rawmeanu = nanmean(u,2);
    rawmeanv = nanmean(v,2); 
    rawstdWindSpd = nanstd( rawWindSpd,0,2 );
    rawGustWindSpd = max( rawWindSpd,[],2 );

    rawmeanWindDir = nanmean( atan2(u,v),2 ); 
    rawstdWindDir = nanstd( atan2(u,v),0,2 );
    
%     rawWindSpd = sqrt( u.^2 + v.^2 );
%     rawmeanWindSpd = nanmean( rawWindSpd, 1 ); 
%     rawmeanu = nanmean(u,1);
%     rawmeanv = nanmean(v,1); 
%     rawstdWindSpd = nanstd( rawWindSpd,0,1 );
%     rawGustWindSpd = max( rawWindSpd,[],1 );
% 
%     rawmeanWindDir = nanmean( atan2(u,v),1 ); 
%     rawstdWindDir = nanstd( atan2(u,v),0,1 );

   
    %% burst loop for wind stress 
    
    window = 128;
    noverlap = window/2;
    nffft = [];
    
    for bi=1:bursts,
        
        
         
        %% replace bad points with burst mean
        %rawWindSpd(bi, isnan(rawWindSpd(bi,:))) = rawmeanWindSpd(bi); 
        %u(bi, isnan(u(bi,:))) = rawmeanu(bi); 
        %v(bi, isnan(v(bi,:))) = rawmeanv(bi); 
   
        
        %% spectra
        [speedpsd f] = pwelch( detrend(rawWindSpd(bi,:)), window, noverlap, nffft, rate ); 
        [upsd f] = pwelch( detrend(u(bi,:)), window, noverlap, nffft, rate ); 
        [vpsd f] = pwelch( detrend(v(bi,:)), window, noverlap, nffft, rate ); 

        figure(1), 
        loglog(f,upsd,'r',f,vpsd,'g'), drawnow, hold on
        loglog(f,speedpsd,'k')

        rawpsd(bi,:) = speedpsd;
        rawupsd(bi,:) = upsd;
        rawvpsd(bi,:) = vpsd;
    
        %% inertial dissipation
        % assume neutral conditions, apply Yelland et al (1994) eq 4 & 5
        K = 0.55 ; % Kolmogorov const (field value)
        kv = 0.4 ; % von Karman const
        z = 9; % measurement height (m) 
        finds = find( f > fmin & f < fmax );
        [speedfit stats] = robustfit(f(finds).^(-5/3),speedpsd(finds),'bisquare',1,'off');
        fitu = robustfit(f(finds).^(-5/3),upsd(finds),'bisquare',1,'off');
        fitv = robustfit(f(finds).^(-5/3),vpsd(finds),'bisquare',1,'off');
        epsilon(bi) =  ( speedfit ./ ( ( rawmeanWindSpd(bi) ./ (2*pi) ).^(2/3)  .* K ) )^(3/2);
        epsilonerrorhigh(bi) = ( speedfit + stats.se ./ ( ( rawmeanWindSpd(bi) ./ (2*pi) ).^(2/3)  .* K ) )^(3/2);
        epsilonerrorlow(bi) = ( speedfit - stats.se ./ ( ( rawmeanWindSpd(bi) ./ (2*pi) ).^(2/3)  .* K ) )^(3/2);
        epsilonu(bi) =  ( fitu ./ ( ( rawmeanWindSpd(bi) ./ (2*pi) ).^(2/3)  .* K ) )^(3/2);
        epsilonv(bi) =  ( fitv ./ ( ( rawmeanWindSpd(bi) ./ (2*pi) ).^(2/3)  .* K ) )^(3/2);
        sigma(bi) = stats.ols_s;

%         if sigma(bi)>0.1, loglog(f,speedpsd,'color',[0 0 0]),
%         else
%         end

                
    end
    
    ustar = (kv * epsilon * z ).^(1/3);  % assumes neutral
    ustarerrorhigh = (kv * epsilonerrorhigh * z ).^(1/3);
    ustarerrorlow = (kv * epsilonerrorlow * z ).^(1/3);

    
    loglog(f(finds),f(finds).^(-5/3),'c-','linewidth',4)
    title(datestr(filestarttime))
    
    % quality control
    %ustar(sigma > 0.1 ) = NaN;

    
    %% save & print
    
    save(files(fi).name(1:40))
    
    print('-dpng',[ files(fi).name(1:40) '.png'])

end



%% summarize & QC results 
clear all, close all
!rm *QC*
files = dir(['Robertson_SonicAnemometer*.mat']);

K = 0.55 ; % Kolmogorov const (field value)
kv = 0.4 ; % von Karman const
z = 9; % measurement height (m) 

for fi = 1:length(files), 
    
    load(files(fi).name,'time','ustar*','*Spd','epsilon*')
    
    ustar = (kv * epsilon * z ).^(1/3);  % assumes neutral

    ustar( ustarerrorhigh>2*ustar ) = NaN;
    ustar( ustar>5 ) = NaN;
    
    if sum(isnan(ustar))<length(ustar),
    dragcoef = regress(ustar'.^2,rawmeanWindSpd.^2)
    
    figure(1),
    subplot(2,1,1),
    errorbar(time,rawmeanWindSpd,-rawstdWindSpd,rawstdWindSpd,'o'), datetick, hold on
    set(gca,'YLim',[0 20]), grid
    ylabel('U_9 [m/s]')
    subplot(2,1,2),
    errorbar(time,ustar,ustar-ustarerrorlow,ustar-ustarerrorhigh,'o'), datetick, hold on
    set(gca,'YLim',[0 3]), grid
    ylabel('u_* [m/s]')


    figure(2),
    plot(rawmeanWindSpd,1.3*ustar.^2,'o'), hold on
    plot([1:20],1.3*dragcoef*[1:20].^2,'k-'), hold on
    axis([0 20 0 5]), grid
    ylabel('\tau [Pa]'),
    xlabel('U_9 [m/s]'),
    legend('dissipation','equiv C_D')

    save([files(fi).name 'QC.mat'])
    else end
end
figure(1), print -dpdf JdFwinds.pdf
figure(2), print -dpdf JdFwindstress.pdf




%% incorporate GPS to subtract vessel speed (5 min bursts)
% % assumes GPS start time is same as sonic SDR 
% 
% if isempty(dir(['Fireboat_GPS_' date '.mat'])), 
%     [gi valid utcdate utctime localdate localtime miliseconds lat northsouth lon eastwest altitude speed heading] = textread([fpath date '/Fireboat_GPS_' date '.csv'],'%n%s%s%s%s%s%s%n%s%n%s%n%n%n','delimiter',',','headerlines',1);
%     for gi = 1:length(lat),
%         gpstime(gi) = datenum([char(utcdate(gi)) ' ' char(utctime(gi)) ] );
%     end
%     lon = -lon;
%     speed = speed * .2777; % convert from Km/h to m/s
%     save(['Fireboat_GPS_' date '.mat'],'gpstime','lat','lon','speed','heading','date')
% else
%     load(['Fireboat_GPS_' date '.mat'])
% end
% % transfrom speed + heading to components
% speedeast = speed .* sin( deg2rad(heading) ); % eastward velocity (true)
% speadnorth = speed .* cos( deg2rad(heading) ); % northward velocity (true)
% 
% % reshape GPS five minute bursts, same as sonic SDR files  ** COULD INTERP
% rate = 5; % Hz
% gi = length(lat);  % number of raw GPS point
% gpswindow = rate*burstlength;  % number of GPS points to process for individual spectra
% nwindows = floor(gi/gpswindow); % number of individual windows 
% gpstime = mean(reshape( gpstime(1:(nwindows*gpswindow)),  gpswindow , nwindows)); 
% meanlat = mean(reshape( lat(1:(nwindows*gpswindow)),  gpswindow , nwindows)); 
% meanlon = mean(reshape( lon(1:(nwindows*gpswindow)),  gpswindow , nwindows)); 
% meanvesselspd = mean(reshape( speed(1:(nwindows*gpswindow)),  gpswindow , nwindows));
% meanvesseldir = mean(reshape( heading(1:(nwindows*gpswindow)),  gpswindow , nwindows)); 

%% interpolation (and averaging)
% 
% ustar = interp1(linspace(min(gpstime),max(gpstime),length(ustar)),ustar,gpstime);
% rawmeanWindSpd = interp1(linspace(min(gpstime),max(gpstime),length(rawmeanWindSpd)),rawmeanWindSpd,gpstime);
% rawmeanu = interp1(linspace(min(gpstime),max(gpstime),length(rawmeanu)),rawmeanu,gpstime);
% rawmeanv = interp1(linspace(min(gpstime),max(gpstime),length(rawmeanv)),rawmeanv,gpstime);
% 
% 
% %% correct wind spd (by component)
% % since sonic component are relative to the boat, the correction is 
% % purely in v (positive towards bow)
% 
% for fi=1:min([length(meanvesselspd) length(rawmeanv)])
%     correctedmeanv(fi) = rawmeanv(fi) - meanvesselspd(fi); 
%     cormeanWindSpd(fi) = sqrt( rawmeanu(fi).^2 + correctedmeanv(fi).^2 );
% end
% 
% %cormeanWindSpd = rawmeanWindSpd;  % testing only
% %mismatch = length(rawmeanv) - length(meanvesselspd) 
% 
% 
% %% correct wind to 10 m reference height (Hoffman, 2005)
% 
% z = 3;
% c0 = 3.7; c1 = -1.165;  a = 0.032; k = 1; %k = 0.4 or 2;
% z0 = z .* exp( -(c0 + c1 * log( (a .* k^2 .* cormeanWindSpd.^2)./(9.8 * z) ) )  );
% U10 = log(10./z0) ./ log(z./z0) .* cormeanWindSpd;
% 
% figure(2), clf
% plot(cormeanWindSpd,U10,'kx'), hold on, 
% xlabel('U_3 [m/s]'), ylabel('U_{10} [m/s]'),
% axis([ 0 15 0 15]), grid
% print('-dpng',['U10correction_' date '.png'])
% 
% 
% %% drag coef 
% 
% rho = 1.3; % kg/m^3
% dragcoef = robustfit(U10.^2,ustar(1:length(U10)).^2,'bisquare',1,'off')
% 
% figure(3), clf
% plot(U10,ustar(1:length(U10)),'kx'), hold on, xlabel('U_{10} [m/s]'), ylabel('u_* [m/s]')
% plot([0:15],sqrt(dragcoef.*[0:15].^2),'r.')
% print('-dpng',['Ustar-U10_' date '.png'])
% 
% 
% figure(4), clf
% plot(U10, rho*ustar(1:length(U10)).^2,'kx'), hold on, xlabel('U_{10} [m/s]'), ylabel('\tau = \rho u_*^2 [Pa]')
% plot([0:15],rho*dragcoef.*[0:15].^2,'r.'), 
% legend('from dissipation',['C_d = ' num2str(dragcoef,2)]) 
% print('-dpng',['WindStress_' date '.png'])
% 
% 
% 
% %%
% 
% save(['SonicAnemometer_' date],'U10','ustar','gpstime','dragcoef','meanlat','meanlon','meanvessel*')
